﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Text;

using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace DCITS.OpenPlatform.InvVeriDemo
{
    class Program
    {
        static readonly string openPlatformUrl = "https://open.leshui365.com";
        static void Main(string[] args)
        {
            var myAppKey = "4d2031ce09434236a8186f9879aaf006";
            var myAppSecret = "82d20a36-b110-4201-bffc-401592c02a96";

            var foregroundColor = Console.ForegroundColor;

            Console.WriteLine("开始获取Token...");
            var rtv = GetToken(myAppKey, myAppSecret);
            if (rtv.Success)
            {
                Console.WriteLine("Token获取成功");
                Console.WriteLine("开始查询发票（发票代码：1100173320；发票号码：35867761）...");
                var invoiceArgs = new Dictionary<string, string>
                {
                    {"invoiceCode", "1100173320"},
                    {"invoiceNumber", "35867761"},
                    {"billTime", "2018-01-10"},
                    {"checkCode", "686544"},
                    {"token", rtv.Data}
                };

                rtv = GetInvoiceInfo(JsonConvert.SerializeObject(invoiceArgs));
                if (rtv.Success)
                {
                    Console.WriteLine("查询成功，发票信息如下：");
                    Console.ForegroundColor = ConsoleColor.Green;
                    Console.WriteLine(rtv.Data);
                    Console.ForegroundColor = foregroundColor;
                    var rtvJson = (JObject)JsonConvert.DeserializeObject(rtv.Data);
                    if (rtvJson["RtnCode"].ToString() == "00")
                    {
                        Console.WriteLine("解析发票信息：");
                        var invoiceJson = (JObject)JsonConvert.DeserializeObject(rtvJson["invoiceResult"].ToString());
                        Console.WriteLine("发票代码：{0}", invoiceJson["invoiceNumber"].ToString());
                    }
                }
                else
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine("查询发票时发生错误:{0}", rtv.Message);
                    Console.ForegroundColor = foregroundColor;
                }
            }
            else
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("请求Token时发生错误:{0}", rtv.Message);
                Console.ForegroundColor = foregroundColor;
            }

            Console.Read();
        }

        /// <summary>
        /// 获取授权码。
        /// </summary>
        /// <param name="appKey">客户应用key：需向开放平台申请</param>
        /// <param name="appSecret">客户应用密钥：需向开放平台申请</param>
        /// <returns></returns>
        static ReturnValue<string> GetToken(string appKey, string appSecret)
        {
            var rtv = new ReturnValue<string> { Success = false };

            try
            {

                var url = string.Format("{0}/getToken?appKey={1}&appSecret={2}", openPlatformUrl, appKey, appSecret);

                var client = new HttpClient();
                var task = client.GetStringAsync(url);
                task.Wait();

                rtv.Data = ((JObject)JsonConvert.DeserializeObject(task.Result))["token"].ToString();
                rtv.Success = true;
            }
            catch (AggregateException ex)
            {
                rtv.Message = ex.InnerException.Message;
            }
            catch (Exception ex)
            {
                rtv.Message = ex.Message;
            }

            return rtv;
        }

        static ReturnValue<string> GetInvoiceInfo(string invoiceParams)
        {
            var rtv = new ReturnValue<string> { Success = false };

            try
            {
                var url = openPlatformUrl + "/api/invoiceInfoForCom";
                var httpContent = new StringContent(invoiceParams, Encoding.UTF8, "application/json");

                var client = new HttpClient();
                var postTask = client.PostAsync(url, httpContent);
                postTask.Wait();

                var readTask = postTask.Result.Content.ReadAsStringAsync();
                readTask.Wait();

                rtv.Data = readTask.Result;
                rtv.Success = true;
            }
            catch (AggregateException ex)
            {
                rtv.Message = ex.InnerException.Message;
            }
            catch (Exception ex)
            {
                rtv.Message = ex.Message;
            }

            return rtv;
        }
    }
}
